//
//  ViewController.swift
//  DigitalInput
//
//  Created by Phidgets on 2018-02-06.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {
    
    let ch = DigitalInput()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    @IBOutlet var stateLabel: UILabel!
    @IBOutlet var powerSupplyStack: UIStackView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.stateChange.addHandler(statechange_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let deviceID = try ch.getDeviceID()
            
            DispatchQueue.main.async {
                self.stateLabel.isHidden = false
                if(deviceID == DeviceID.PN_DAQ1400){
                        self.powerSupplyStack.isHidden = false
                }
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        stateHidden(hide: false)
    }
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stateLabel.isHidden = true
                self.powerSupplyStack.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
        stateHidden(hide: true)
    }
    func statechange_handler(sender: DigitalInput, state: Bool){
        stateUpdate(state: state)
    }
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    //MARK: GUI Controls
    @IBAction func setPowerSupply(_ sender: Any) {
        let segment = sender as! UISegmentedControl
        do{
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setPowerSupply(PowerSupply.off)
                break
            case 1:
                try ch.setPowerSupply(PowerSupply.volts_12)
                break
            case 2:
                try ch.setPowerSupply(PowerSupply.volts_24)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    @IBAction func setInputMode(_ sender: Any) {
        let segment = sender as! UISegmentedControl
        do{
            switch(segment.selectedSegmentIndex){
            case 0:
                try ch.setInputMode(InputMode.NPN)
                break
            case 1:
                try ch.setInputMode(InputMode.PNP)
                break
            default:
                break
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    
    //MARK: UI Updates
    func stateHidden(hide:Bool){
        DispatchQueue.main.async {
            self.stateLabel.isHidden = hide
        }
    }
    
    func stateUpdate(state:Bool){
        DispatchQueue.main.async {
            if(state){
                self.stateLabel.text = "True"
            }
            else{
                self.stateLabel.text = "False"
            }
        }
    }
    
    
    //MARK: Navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    
}
